function [eX, eY, time, trials] = eppmatrix (c, k)

% @CORTEX/EPPMATRIX  returns a matrix of EPP values from a @cortex object, 
%                    for the trials and over the epoch specified by a @key
%
% Syntax: [EPPX,EPPY,TIME,TRIALS] = EPPMATRIX(COBJ,KEY);
%
% where
% COBJ:   the @cortex object
% KEY:    one key from a @key object
% EPPX/Y: matrix of EPP values (one row per trial, one column per sample;
%         values are time-locked on the cue(s) specified in the key
% TIME:   the corresponding time scale
% TRIALS: a vector of trial numbers 
%
% Last modified: 07 Feb 06

% Created on 23 June 04
% 28 Oct 04: cosmetics

e = c.epp;
if ~e.rate, error ('object does not contain EPP data'); end
errifnotsinglekey (k);
yestrials = trialset (c, k);
% % if nargin > 2, errifnotscalar (Ch); end
% % switch ch
% %    case 1
% %       epps = c.epp.ch1;
% %    case 2
% %       epps = c.epp.ch2;
% %    otherwise
% %       error ('invalid channel number');
% % end

% this subtraction finds the number of EPP pairs per trial
epps_per_trial = diff (e.ptr);
% array of trials that do contain EPPS
trials_with_epps = find (epps_per_trial);

yestrials_with_epps = intersect (trials_with_epps, yestrials);
if length (yestrials_with_epps) < length (yestrials)
   warning ('among the requested trials, the following do not have EPPs:');
   disp (setdiff(yestrials,trials_with_epps));
end
samples_per_row = fix ((k.finish-k.start)/e.rate);
eX = zeros (length (yestrials_with_epps), samples_per_row);
eY = zeros (length (yestrials_with_epps), samples_per_row);
instance = 0;
trials  = [];
cueless = [];
winless = [];

for trial=yestrials_with_epps
   first = e.ptr(trial);
   last  = e.ptr(trial+1)-1;
   samples = epps_per_trial(trial);
   % of all requested cues, find those that are actually present
   % in the current trial. THIS MAY BE SLOW!
   cues_found = intersect (k.cues, c.codes (trial, :));
   % if no cue event is found add trial to cueless
   if isempty (cues_found), cueless = [cueless trial]; end
   % for each cue event found
   for j=cues_found
      % there should NOT be more than one instance of the same cue event!
      cue_index = find (c.codes (trial, :) == j);
      if length (cue_index)>1
         disp (['only the first instance of event ', num2str(j), ' will be considered']);
         cue_index = cue_index(1);
      end
      
      kt = c.times (trial, cue_index);
      start_time = kt + k.start;
      end_time   = kt + k.finish;
      start_from_first = start_time - e.start(trial);
      if (start_from_first < 0 | (end_time > (start_time + samples * e.rate)))
         winless = [winless trial];
      else
         instance = instance + 1;
         trials = [trials; trial]; 
         findx = first + fix (start_from_first/e.rate);
         lindx = findx + samples_per_row-1;
			eX (instance, :) = double(e.ch1 (findx:lindx))';
			eY (instance, :) = double(e.ch2 (findx:lindx))';
% %          eppm (instance, :) = double(epps (findx:lindx))';
      end
   end
end
eX = eX (1:instance, :);
eY = eY (1:instance, :);

% % eppm = eppm (1:instance, :);
time = k.start:e.rate:k.finish-e.rate;

if ~isempty (cueless)
   warning ('could not retrieve all requested trials;');
   disp ('no cue events found in the following trial(s):');
   disp (cueless);
end
if ~isempty (winless)
   warning ('could not retrieve all requested trials;');
   disp ('time window exceeds boundaries of the following trial(s)');
   disp (winless);
end