function [spmat, uni, grp] = spikematrix (sp, win, binsize, varargin)

%@SPIX/SPIKEMATRIX   matrix of spikes from a @spix object
%
%    SPMAT = SPIKEMATRIX(SP,EPOCH,BIN) returns a matrix of ones (spike
%    occurred) and zeros (no spike) from the SP @spix object for all
%    currently selected units and for each of the currently selected
%    keys. If EPOCH is a scalar, then the spike matrix covers a BIN-
%    ms-long epoch starting EPOCH ms after the cue event. If EPOCH is
%    a 2-element array, then the 2 values define the start and end
%    points of the epoch (again, in ms relative to the cue event), and
%    BIN (although still required) is ignored.
%
%    EPOCH can also be one of the following strings:
%    'all': selects the entire time window contained in SP
%    'pre': selects the epoch preceding the cue event 
%    'pos': selects the epoch following the cue event
%    'var': instead of time-locking to the cue event, previously
%           defined, cell-by-cell latencies are used to align trials;
%           if 'var' is used, firing rates are computed on a single
%           epoch of BINSIZE duration
%
%    SPMAT is an M-by-N matrix where M is the number of analyzed trial
%    epochs, and N is the epoch length in ms.
%
%    SPMAT = SPIKEMATRIX(...,RESTRICTIONS) returns only trial epochs
%    specified by RESTRICTIONS. See online help for RESTRICTIONS
%    usage.
%
%    [SPMAT,UNITS,CRIT] = SPIKEMATRIX(...) also returns an array of
%    UNITS from which each trial epoch derives, and CRIT, an array of
%    keys, combos, or levels matching each trial.
%
%Last modified: 29 Nov 04

% changes since 12 Sep 03:
% 29 Nov 04: help text revised

[yest, uni, grp] = trialset (sp, varargin{:});
% [included_trials, i] = sort (sp.trial(yest));

if ischar(win)
   switch minifield (win)
      case 'var'
         if ~isnumscalar (binsize), error ('with variable latency analysis, BINSIZE must be a numeric scalar'); end
         for u = unique (uni)'
            % variable latency computations
            span = sp.latency(u) - sp.offset : sp.latency(u) - sp.offset + binsize - 1;
            idx = yest (uni==u);
            chunk(idx, :) = sp.spix (idx, span);
         end
         spmat = full(chunk(yest,:));
      case 'all'
         spmat = full(sp.spix (yest,:));
      case 'pos'
         spmat = full(sp.spix (yest, sp.offset:end));
      case 'pre'
         spmat = full(sp.spix (yest, 1:sp.offset));
      otherwise
         error ('bad parameter');
   end
elseif isnumeric(win)
   if length(win)==1
      if ischar (binsize)
         if ~strcmp (minifield(binsize), 'all'), error ('bad parameter'); end
         win(2) = size (sp.spix, 2);
      elseif isnumscalar (binsize)
         win(2) = win + binsize -1;
      else
         error ('bad parameter');
      end
   end
   spmat = full(sp.spix (yest, win(1)-sp.offset:win(2)-sp.offset));
else
   error ('bad parameter');
end
