function [frates, uni, grp] = trialfiringrate (sp, win, binsize, varargin)

%@SPIX/TRIALFIRINGRATE Trial-by-trial firing rates from a @spix object.
%
%    FRATES = TRIALFIRINGRATE(SP,EPOCH,BINSIZE) returns spike/second
%    rates from the SP @spix object for each of the currently selected
%    units and for each of the trials specified by the currently
%    selected keys. If EPOCH is a scalar, then firing rates are
%    computed over a BINSIZE-ms-long time window starting EPOCH ms
%    after the cue event. If EPOCH is a 2-element array, then EPOCH(1)
%    and EPOCH(2) define, respectively, the first and the last ms of
%    the analysis window (again, relative to the cue event), which is
%    divided in BINSIZE-ms-long bins. Separate firing rates, one per
%    bin, are then returned. If the epoch is not a multiple of
%    binsize, trailing milliseconds are ignored.
%
%    EPOCH can also be one of the following strings:
%    'all': selects the entire time window contained in SP
%    'pre': selects the epoch preceding the cue event 
%    'pos': selects the epoch following the cue event
%    'var': instead of time-locking to the cue event, previously
%           defined, cell-by-cell latencies are used to align trials;
%           if 'var' is used, firing rates are computed on a single
%           epoch of BINSIZE duration
%
%    FRATES is an M-by-N matrix, where M is the number of trials
%    included in the analysis, and N is the number of bins.
%
%    FRATES = TRIALFIRINGRATE(...,RESTRICTIONS) returns only data
%    computed on trial epochs specified by RESTRICTIONS. See online
%    help for RESTRICTIONS usage.
%
%    [FRATES,UNITS,GROUPS] = TRIALFIRINGRATE(...) also returns UNITS,
%    an array that contains the unit# from which each firing rate was
%    computed, and GROUPS, a similar list containing the key or combo
%    used to compute each firing rate. It follows that:
%    length(units)==length(grps)==size(frates,1)
%
% Last modified: 27 Nov 04

% 24 dec 02: last unnoted changes
% 16 apr 03: help text
% 17 Apr 03: call to truncspm
% 23 Apr 03: call to trialset replaces findyestrials
% 18 Aug 03: complete change of concept!
% 27 Nov 04: help text revised

[chunk uni, grp] = spikematrix (sp, win, binsize, varargin{:});
[spmat, instances, winsize, numofbins] = truncspm (chunk, binsize);
spmat = reshape(spmat, instances, binsize, numofbins);
frates = squeeze (sum (spmat, 2) / binsize * 1000);
