function varargout = quickie(varargin)

% QUICKIE  interactive display of density plots
%
%    QUICKIE(COBJ,CLUSTER,KEYS)   creates a GUI that displays up to 4
%    spike density plots for the specified CLUSTER in the COBJ @cortex
%    object. KEY is a @key object containing any number of keys,
%    provided that all keys specify epochs of identical length.  The
%    list of available keys is shown in the upper right side of the
%    figure. In order to display a density plot for a key, first
%    select one the colored radio buttons, then click on the desired
%    key. Use the checkbox to the right of each radio button to
%    show/hide a plot. Use the 'Reset plots' button to clear all
%    plots. Edit the 'Kernel width' text box to change the standard
%    deviation (in ms) of the smoothing gaussian function. Larger
%    values yield smoother but less accurate plots. Checking 'Fixed Y
%    axis limis' will cause the program to determine the Y scale so as
%    to accommodate plots for all available keys. The yellow and pink
%    shaded areas represent epochs for instant analysis of the mean
%    firing rates. The E1 and E2 boxes in the bottom-right part of the
%    panel let the user set the starting point (relative to the key's
%    cue event) and the duration in ms of the epoch, and show average
%    firing rates for each of the currently selected keys to the left.
%    Epochs can be moved (but not resized) by dragging the yellow and
%    pink rectangles directly on the plot areas. The 4 boxes to the
%    extreme right are the p-values of paired t-tests comparing firing
%    rates across epochs (typically used to show cell responsivity).
%    The 2 boxes below the shaded E1 and E2 areas contain the p-values
%    for a 1-way ANOVA comparing firing rates across keys (i.e,
%    experimental conditions).
%
%    QUICKIE(SPIX)  same as above, but showing plots for all units
%    currently available in the SPIX @spix object, using the current
%    set of embedded keys. The bottom left part of the figure has
%    controls to browse the available units.
%
%Last modified: 15 Feb 06

% 29 Oct 04: major revisions
%  7 Nov 04: GiuCoDefaultValues implemented
% 15 Feb 05: bugs fixed in UpdateNumbers():
%            - tr forced vertical
%            - all-zero or single-observation data caught
% 18 Feb 05: small bug in UpdateNumbers()
% 15 Feb 06: getdefs introduced

% Last Modified by GUIDE v2.5 28-Oct-2004 17:43:14

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @quickie_OpeningFcn, ...
                   'gui_OutputFcn',  @quickie_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin & isstr(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before quickie is made visible.
function quickie_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to quickie (see VARARGIN)

% Choose default command line output for quickie
handles.output = hObject;

ob = varargin{1};
if isa (ob, 'spix')
   handles.spix = true;
   handles.sp = verbose(ob, false);
   handles.k  = ob.key;
   set (handles.charting, 'name', ['Quickie for ' ob.name ' @spix object']);
   set (handles.unitname, 'string', handles.sp.uni(1).label);
elseif isa (ob, 'cortex')
   handles.spix = false;
   handles.cobj = ob;
   errifnotscalar (varargin{2}, 'CLUSTER');
   handles.cell = varargin{2};
   errifnotkey (varargin{3}, 'KEYS');
   handles.k = varargin{3};
   set (handles.charting, 'name', ['Quickie for cell: ' handles.cobj.name '.' num2str(handles.cell)]);
   set ([ ...
         handles.cmdFirst ...
         handles.cmdLast ...
         handles.cmdPrev ...
         handles.cmdNext ...
         handles.unit ...
         handles.unitname], 'visible', 'off');
else
   error ('first argument is either a @cortex or a @spix object')
end

% populate keylist
set (handles.keylist, 'String', handles.k.lab);

% set defaults
% % global GiuCoDefaultValues
% % if isempty (GiuCoDefaultValues), readefs; end
% % GiuCoDefaultValues.quickie;
d = getdefs ('quickie');

% general parameters
set (handles.ksize, 'string', d.kwidth);
set (handles.chkylim, 'value', d.fixylim);

% epoch1
p = d.epoch1start;
b = d.epoch1size;
set (handles.start1, 'string', p);
set (handles.len1, 'string', b);
handles.pa(1) = patch ([p p+b p+b p], [0 0 1 1], [1 1 .7]);
set (handles.chkE1, 'value', d.epoch1on);
if d.epoch1on, s='on'; else s='off'; end
set (handles.pa(1), 'visible', s);

% epoch2
p = d.epoch2start;
b = d.epoch2size;
set (handles.start2, 'string', p);
set (handles.len2, 'string', b);
handles.pa(2) = patch ([p p+b p+b p], [0 0 1 1], [1 .85 .85]);
set (handles.chkE2, 'value', d.epoch2on);
if d.epoch2on, s='on'; else s='off'; end
set (handles.pa(2), 'visible', s);

set (handles.pa, 'linestyle', 'none');

% create plots
win = [handles.k(1).start handles.k(1).finish-1];
hold on;
cols = 'brgk';
for i=1:4
   handles.plo(i) = plot (win(1):win(2), zeros (1, diff(win)+1), cols(i));
end

% create extra handles variables to add via guidata
handles.rb(1) = handles.radiobutton1;
handles.rb(2) = handles.radiobutton2;
handles.rb(3) = handles.radiobutton3;
handles.rb(4) = handles.radiobutton4;
handles.arb = 1;
set (handles.rb(1), 'value', 1);

handles.cplo(1) = handles.chkplo1;
handles.cplo(2) = handles.chkplo2;
handles.cplo(3) = handles.chkplo3;
handles.cplo(4) = handles.chkplo4;
handles.chkplo = [0 0 0 0];

handles.epoch(1) = get (handles.chkE1, 'value');
handles.epoch(2) = get (handles.chkE2, 'value');

handles.start(1) = handles.start1;
handles.start(2) = handles.start2;
handles.len(1)   = handles.len1;
handles.len(2)   = handles.len2;

handles.means(1,1) = handles.mean1;
handles.means(2,1) = handles.mean2;
handles.means(3,1) = handles.mean3;
handles.means(4,1) = handles.mean4;
handles.means(1,2) = handles.meanB1;
handles.means(2,2) = handles.meanB2;
handles.means(3,2) = handles.meanB3;
handles.means(4,2) = handles.meanB4;

handles.p(1) = handles.pp1;
handles.p(2) = handles.pp2;
handles.p(3) = handles.pp3;
handles.p(4) = handles.pp4;

handles.spm={[], [], [], []};

handles.epo = 1;
handles.padown = 0;
handles.cpx = 0;

% Update handles structure
guidata(hObject, handles);
set (handles.chart1, 'xlim', win);
if d.fixylim, recalcylim(handles); end
UpdateChart (handles);
UpdateNumbers (handles);
% UIWAIT makes quickie wait for user response (see UIRESUME)
% uiwait(handles.charting);

% --- Outputs from this function are returned to the command line.
function varargout = quickie_OutputFcn(hObject, eventdata, handles)
% varargout  cell array for returning output args (see VARARGOUT);
% Get default command line output from handles structure
varargout{1} = handles.output;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function keylist_Callback(hObject, eventdata, handles)

ck = get(hObject,'value');
for j=1:4
   if strcmp (get (handles.rb(j), 'string'), handles.k(ck).label)
      beep;
      return
   end
end
set (handles.cplo (handles.arb), 'value', 1);
handles.chkplo (handles.arb) = 1;
set (handles.rb (handles.arb), 'string', handles.k(ck).label);
if handles.spix
   handles.spm{handles.arb} = spikematrix (handles.sp, 'all', 'all', 'uni', uivalue(handles.unit), 'key', ck);
else
   handles.spm{handles.arb} = spikematrix (handles.cobj, handles.cell, handles.k(ck));
end
guidata(handles.chart1, handles);
UpdateChart (handles);
UpdateNumbers (handles, handles.arb);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% --- Redraw Chart1 based on the selected key
function UpdateChart (handles)

ksize = uivalue(handles.ksize);
maxde = 0;
for ck = find (handles.chkplo)
   de = density (handles.spm{ck}, ksize, 'gauss');
   if isempty (de)
      set (handles.plo(ck), 'visible', 'off');
   else
      maxde = max (maxde, max(de));
      set (handles.plo(ck), 'ydata', de);
      set (handles.plo(ck), 'visible', 'on');
   end
end
if ~get(handles.chkylim, 'value')
   ylim2 = max(10, ceil(maxde/10)*10);
   set (handles.chart1, 'ylim',[0 ylim2]);
   set (handles.pa, 'ydata', [0 0 ylim2 ylim2]);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function UpdateNumbers (handles, series)

[sta, fin, xlim] = getcurrentbin (handles);
X = [];
g = [];
cg = 0;
for ck = 1:4
   if handles.chkplo (ck)
      spm = handles.spm{ck};
      if isempty (spm)
         set (handles.means(ck,:), 'string', 'N/A');
      else
         tX = [];
         cg = cg + 1;
         for ep = 1:2
            epoch = sta(ep)-xlim(1):fin(ep)-xlim(1)-1;
            tr = trialfiringrate (spm (:, epoch));
            set (handles.means(ck,ep), 'string', sprintf ('%.2f', mean(tr)));
            tX = [tX, tr'];
         end
         tdata = diff (tX,1,2);
         if all(tdata==0) | length(tdata)==1
            set (handles.p(ck), 'string', 'NaN');
         else
            try
               [h p] = ttest (tdata);
               set (handles.p(ck), 'string', sprintf ('%1.6f', p));
            catch
               set (handles.p(ck), 'string', 'N/A');
            end
         end
         X = [X; tX];
         g = [g; repmat(cg,length(tr),1)];
      end
   else
      set (handles.means(ck,:), 'string', []);
      eval(['set(handles.pp' num2str(ck) ',''string'',''N/A'')']);
   end
end

if cg > 1
   p = anova1 (X(:,1), g, 'off');
   set (handles.anovap1, 'string', sprintf ('%1.6f',p));
   p = anova1 (X(:,2), g, 'off');
   set (handles.anovap2, 'string', sprintf ('%1.6f',p));
else
   set (handles.anovap1, 'string', 'N/A');
   set (handles.anovap2, 'string', 'N/A');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function charting_WindowButtonDownFcn(hObject, eventdata, handles)

cp = round(get (handles.chart1, 'currentpoint'));
cpx = cp(1,1); cpy = cp(1,2);
paY = get(handles.pa(1), 'ydata');
if cpy<paY(1) | cpy>paY(3), return, end

pa = get(handles.pa(1), 'xdata');
if cpx>pa(1) & cpx<pa(2)
   handles.padown = 1;
else
   pa = get(handles.pa(2), 'xdata');
   if cpx>pa(1) & cpx<pa(2)
      handles.padown = 2;
   else
      return
   end
end
handles.cpx = cpx;
guidata(handles.chart1, handles);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function charting_WindowButtonMotionFcn(hObject, eventdata, handles)

selpa = handles.padown;
if ~selpa, return, end
xlim = get(handles.chart1, 'xlim');
pa = get(handles.pa(selpa), 'xdata');
bin = pa(2)-pa(1);
cp = round(get (handles.chart1, 'currentpoint'));
cpx = cp(1,1);
xdiff = cpx-handles.cpx;
if xdiff < 0, nx = max(xlim(1)+1, pa(1)+xdiff);
else, nx = min(xlim(2)-bin, pa(1)+xdiff); end
set(handles.pa(selpa), 'xdata', [nx nx+bin nx+bin nx]);
handles.cpx = cpx;
set (handles.start(selpa), 'string', nx);
guidata(handles.chart1, handles);
drawnow;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function charting_WindowButtonUpFcn(hObject, eventdata, handles)
handles.padown = 0;
guidata(handles.chart1, handles);
UpdateNumbers (handles, 1:4);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function len1_Callback(hObject, eventdata, handles)
if uivalue(hObject)<1, set (hObject, 'string', 1); end
updatepochs (handles, 1);
function len2_Callback(hObject, eventdata, handles)
if uivalue(hObject)<1, set (hObject, 'string', 1); end
updatepochs (handles, 2);
function start1_Callback(hObject, eventdata, handles)
updatepochs (handles, 1);
function start2_Callback(hObject, eventdata, handles)
updatepochs (handles, 2);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function updatepochs (handles, epo)
[sta, fin, xlim] = getcurrentbin (handles);
bin = min (fin(epo)-sta(epo), xlim(2)-xlim(1));
sta = min (xlim(2)-bin, sta(epo));
sta = max (xlim(1)+1, sta);
set (handles.start(epo), 'string', sta);
set (handles.len(epo), 'string', bin);
set (handles.pa(epo), 'xdata', [sta, sta+bin, sta+bin, sta]);
UpdateNumbers (handles, 1:4);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [start, finish, xlim] = getcurrentbin (handles)
start(1) = uivalue (handles.start1);
start(2) = uivalue (handles.start2);
finish(1) = start(1) + uivalue (handles.len1);
finish(2) = start(2) + uivalue (handles.len2);
xlim = get (handles.chart1, 'xlim');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%--- Executes on button press in radiobutton(1:4)
function radiobutton1_Callback(hObject, eventdata, handles)
mutuallyradio (handles, 1);
function radiobutton2_Callback(hObject, eventdata, handles)
mutuallyradio (handles, 2);
function radiobutton3_Callback(hObject, eventdata, handles)
mutuallyradio (handles, 3);
function radiobutton4_Callback(hObject, eventdata, handles)
mutuallyradio (handles, 4);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function mutuallyradio (handles, acceso)
set (handles.rb, 'value', 0);
set (handles.rb(acceso), 'value', 1);
handles.arb = acceso;
guidata(handles.chart1, handles);

% --- Executes on button press in chkplo(1:4)
function chkplo1_Callback(hObject, eventdata, handles)
manageplochk (handles, 1);
function chkplo2_Callback(hObject, eventdata, handles)
manageplochk (handles, 2);
function chkplo3_Callback(hObject, eventdata, handles)
manageplochk (handles, 3);
function chkplo4_Callback(hObject, eventdata, handles)
manageplochk (handles, 4);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function manageplochk (handles, this)
      
thiscplo = handles.cplo(this);
tval = get(thiscplo, 'value');
if tval & isempty (get (handles.rb(this), 'string'))
   set(thiscplo, 'value', 0);
   return
end
handles.chkplo(this) = tval;
if ~tval, set (handles.plo(this), 'visible', 'off'); end
guidata (handles.chart1, handles);
UpdateChart (handles);
UpdateNumbers (handles, this);

% --- Executes on button press in chkE1-2.
function chkE1_Callback(hObject, eventdata, handles)
if get (hObject, 'value')
   set (handles.pa(1), 'visible', 'on');
else
   set (handles.pa(1), 'visible', 'off');
end
function chkE2_Callback(hObject, eventdata, handles)
if get (hObject, 'value')
   set (handles.pa(2), 'visible', 'on');
else
   set (handles.pa(2), 'visible', 'off');
end

% --- Executes on button press in rbepoch1.
function rbepoch1_Callback(hObject, eventdata, handles)
handles.epo = 1;
set (handles.rbepoch1, 'value', 1);
set (handles.rbepoch2, 'value', 0);
guidata (hObject, handles);

% --- Executes on button press in rbepoch2.
function rbepoch2_Callback(hObject, eventdata, handles)
handles.epo = 2;
set (handles.rbepoch1, 'value', 0);
set (handles.rbepoch2, 'value', 1);
guidata (hObject, handles);

function ksize_Callback(hObject, eventdata, handles)
if get(handles.chkylim, 'value'), recalcylim(handles); end
UpdateChart (handles);

% --- Executes on button press in resetplots.
function resetplots_Callback(hObject, eventdata, handles)

handles.chkplo(:) = 0;
handles.arb = 1;
set (handles.cplo, 'value', 0);
set (handles.rb, 'string', []);
set (handles.rb, 'value', 0);
set (handles.radiobutton1, 'value', 1);
set (handles.plo, 'visible', 'off');
guidata (hObject, handles);
UpdateChart (handles);
UpdateNumbers(handles, 1:4);


% --- Executes on button press in chkylim.
function chkylim_Callback(hObject, eventdata, handles)
if get(hObject, 'value'), recalcylim(handles); end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function recalcylim (handles)
set (handles.charting, 'pointer', 'watch')
ksize = uivalue (handles.ksize);
maxde = 0;
k = handles.k;
if handles.spix
   de = density (handles.sp, 'all', 'all', ksize, 'gauss', 'uni', uivalue(handles.unit), 'key', 'all');
   maxde = max(max(de));
else
   for ck = 1:k.len
      de = density (handles.cobj, handles.cell, ksize, 'gauss', k(ck));
      if ~isempty(de), maxde = max (maxde, max(de)); end
   end
end
ylim2 = max(10, ceil(maxde/10)*10);
set (handles.chart1, 'ylim',[0 ylim2]);
set (handles.pa, 'ydata', [0 0 ylim2 ylim2]);
set(handles.charting, 'pointer', 'arrow');

%%%%%%%%%%%%%%%%%%%%%%%%%
% browsing units in spix
%%%%%%%%%%%%%%%%%%%%%%%%%
function unit_Callback(hObject, eventdata, handles)
cu = uivalue(hObject);
if isempty(cu) | cu<1, set (handles.unit, 'string', 1); end
if cu>handles.sp.uni.count, set (handles.unit, 'string', handles.sp.uni.count); end
changeunit (handles);
% -----------------------------------------------------
function cmdFirst_Callback(hObject, eventdata, handles)
set (handles.unit, 'string', 1);
changeunit (handles);
% -----------------------------------------------------
function cmdPrev_Callback(hObject, eventdata, handles)
cu = uivalue(handles.unit);
if cu==1, return, end
cu = cu - 1;
set (handles.unit, 'string', cu);
changeunit (handles);
% -----------------------------------------------------
function cmdNext_Callback(hObject, eventdata, handles)
cu = uivalue(handles.unit);
if cu==handles.sp.uni.count, return, end
cu = cu + 1;
set (handles.unit, 'string', cu);
changeunit (handles);
% -----------------------------------------------------
function cmdLast_Callback(hObject, eventdata, handles)
set (handles.unit, 'string', handles.sp.uni.count);
changeunit (handles);
% -----------------------------------------------------
function changeunit (handles)
unit = uivalue (handles.unit);
set (handles.unitname, 'string', handles.sp.uni(unit).label);
for i=find(handles.chkplo)
   klab = get (handles.rb(i), 'string');
   handles.spm{i} = spikematrix (handles.sp, 'all', 'all', 'uni', unit, 'key', klab);
end
guidata(handles.chart1, handles);
UpdateChart (handles);
UpdateNumbers(handles, 1:4);

function unitname_Callback(hObject, eventdata, handles)
% lookup unit name in spix?